@@ -48,6 +48,17 @@ SINT32 CAASymCipher::destroy()
 		return E_SUCCESS;
 	}
 
+inline void setRSAFlags(RSA* pRSA)
+	{
+		if(pRSA==NULL)
+			return;
+		pRSA->flags|=RSA_FLAG_THREAD_SAFE;
+		pRSA->flags|=RSA_FLAG_NO_BLINDING;
+#if OPENSSL_VERSION_NUMBER	> 0x0090705fL
+		pRSA->flags|=RSA_FLAG_NO_EXP_CONSTTIME;
+#endif
+	}
+
 /** Decrypts exactly one block which is stored in @c from. 
 	*The result of the decryption is stored in @c to.
 	*@param from one block of cipher text
@@ -55,7 +66,7 @@ SINT32 CAASymCipher::destroy()
 	*@retval E_UNKNOWN in case of an error
 	*@retval E_SUCCESS otherwise
 	*/
-SINT32 CAASymCipher::decrypt(UINT8* from,UINT8* to)
+SINT32 CAASymCipher::decrypt(const UINT8* from,UINT8* to)
 	{
 		if(RSA_private_decrypt(RSA_SIZE,from,to,m_pRSA,RSA_NO_PADDING)==-1)
 			return E_UNKNOWN;
@@ -63,6 +74,37 @@ SINT32 CAASymCipher::decrypt(UINT8* from,UINT8* to)
 			return E_SUCCESS;
 	}
 
+/** Decrypts one OAEP encoded block which is stored in @c from. 
+	*@param from one OAEP encoded block of cipher text
+	*@param to the plain text
+	*@param len on return contains the size of the plaintext
+	*@retval E_UNKNOWN in case of an error
+	*@retval E_SUCCESS otherwise
+	*/
+SINT32 CAASymCipher::decryptOAEP(const UINT8* from,UINT8* to,UINT32* len)
+	{
+		SINT32 ret=RSA_private_decrypt(RSA_SIZE,from,to,m_pRSA,RSA_PKCS1_OAEP_PADDING);
+		if(ret<0)
+			return E_UNKNOWN;
+		*len=ret;
+		return E_SUCCESS;
+	}
+	
+/** Encrypts one block using OAEP padding. 
+	*@param from one of plain text
+	*@param to the OAEP encoded cipher text
+	*@param len on return contains the size of the ciphertext
+	*@retval E_UNKNOWN in case of an error
+	*@retval E_SUCCESS otherwise
+	*/
+SINT32 CAASymCipher::encryptOAEP(const UINT8* from,UINT32 fromlen,UINT8* to,UINT32* len)
+	{
+		SINT32 ret=RSA_public_encrypt(fromlen,from,to,m_pRSA,RSA_PKCS1_OAEP_PADDING);
+		if(ret<0)
+			return E_UNKNOWN;
+		*len=ret;
+		return E_SUCCESS;
+	}
 /** Encrypts exactly one block which is stored in @c from. 
  *The result of the encrpytion is stored in @c to.
 	*@param from one block of plain text
@@ -70,7 +112,7 @@ SINT32 CAASymCipher::decrypt(UINT8* from,UINT8* to)
 	*@retval E_UNKNOWN in case of an error
 	*@retval E_SUCCESS otherwise
 	*/
-SINT32 CAASymCipher::encrypt(UINT8* from,UINT8* to)
+SINT32 CAASymCipher::encrypt(const UINT8* from,UINT8* to)
 	{
 		if(RSA_public_encrypt(RSA_SIZE,from,to,m_pRSA,RSA_NO_PADDING)==-1)
 			return E_UNKNOWN;
@@ -87,6 +129,7 @@ SINT32 CAASymCipher::generateKeyPair(UINT32 size)
 	{
 		RSA_free(m_pRSA);
 		m_pRSA=RSA_generate_key(size,65537,NULL,NULL);
+		setRSAFlags(m_pRSA);
 		if(m_pRSA==NULL)
 			return E_UNKNOWN;
 		else
@@ -108,6 +151,7 @@ SINT32 CAASymCipher::generateKeyPair(UINT32 size)
 	*@see getPublicKeySize()
 	*@see setPublicKey()
 	*/
+	/*
 SINT32 CAASymCipher::getPublicKey(UINT8* buff,UINT32 *len)
 	{
 		if(m_pRSA==NULL||buff==NULL)
@@ -128,7 +172,7 @@ SINT32 CAASymCipher::getPublicKey(UINT8* buff,UINT32 *len)
 		aktIndex+=BN_num_bytes(m_pRSA->e);
 		*len=aktIndex;
 		return E_SUCCESS;
-	}
+	}*/
 
 /** Returns the number of bytes needed to store we public key. This is the number of bytes needed for a
 	* call of getPublicKey().
@@ -136,12 +180,12 @@ SINT32 CAASymCipher::getPublicKey(UINT8* buff,UINT32 *len)
 	*        number of bytes otherwise
 	*@see getPublicKey
 	*/
-SINT32 CAASymCipher::getPublicKeySize()
+/*SINT32 CAASymCipher::getPublicKeySize()
 	{
 		if(m_pRSA==NULL||m_pRSA->n==NULL||m_pRSA->e==NULL)
 			return E_UNKNOWN;
 		return (SINT32)BN_num_bytes(m_pRSA->n)+BN_num_bytes(m_pRSA->e)+4;
-	}
+	}*/
 
 /** Sets the public key to the vaules stored in \c key. The format must match the format described for getPublicKey(). 
 	*@param key byte array which holds the new public key
@@ -150,7 +194,7 @@ SINT32 CAASymCipher::getPublicKeySize()
 	*@retval E_SUCCESS otherwise
 	*@see getPublicKey
 	*/
-SINT32 CAASymCipher::setPublicKey(UINT8* key,UINT32* len)
+/*SINT32 CAASymCipher::setPublicKey(UINT8* key,UINT32* len)
 	{
 		UINT32 aktIndex;
 		UINT32 availBytes;
@@ -185,12 +229,15 @@ SINT32 CAASymCipher::setPublicKey(UINT8* key,UINT32* len)
 		BN_bin2bn(key+aktIndex,size,m_pRSA->e);
 		aktIndex+=size;
 		(*len)=aktIndex;
+		m_pRSA->flags|=RSA_FLAG_THREAD_SAFE;
+		m_pRSA->flags|=RSA_FLAG_NO_BLINDING;
 		return E_SUCCESS;
 _ERROR:
 		RSA_free(m_pRSA);
 		m_pRSA=NULL;
 		return E_UNKNOWN;
 	}
+*/
 
 /** Stores the public key in \c buff as XML. The format is as follows:
 	*
@@ -252,42 +299,6 @@ SINT32 CAASymCipher::getPublicKeyAsDocumentFragment(DOM_DocumentFragment& dFrag)
 		dFrag.appendChild(root);
 		return E_SUCCESS;
 	}
-/*
-//XML Decode...
-static void sRSAKeyParamValueHandler(XMLElement &elem, void *userData)
-{
-	UINT8 buff[4096];
-	int len=(int)elem.ReadData((char*)buff,4096);
-	
-	UINT32 decLen=4096;
-	UINT8 decBuff[4096];
-	CABase64::decode(buff,len,decBuff,&decLen);
-	
-	RSA* tmpRSA=(RSA*)userData;
-	switch(elem.GetName()[0])
-		{
-			case 'E':
-				if(tmpRSA->e!=NULL)
-					BN_free(tmpRSA->e);
-				tmpRSA->e=BN_bin2bn((unsigned char*)decBuff,decLen,NULL);
-			break;
-			case 'M':
-				if(tmpRSA->n!=NULL)
-					BN_free(tmpRSA->n);
-				tmpRSA->n=BN_bin2bn((unsigned char*)decBuff,decLen,NULL);
-			break;
-		}	
-}
-
-static void sRSAKeyValueHandler(XMLElement &elem, void *userData)
-{
-	XMLHandler handlers[] = {
-	XMLHandler("Modulus",sRSAKeyParamValueHandler),
-	XMLHandler("Exponent",sRSAKeyParamValueHandler),
-	XMLHandler::END};
-		elem.Process(handlers, userData);
-}
-*/
 /** Sets the public key to the values stored in \c key. 
 	* The format must match the format XML described for getPublicKeyAsXML(). 
 	*@param key byte array which holds the new public key
@@ -296,7 +307,7 @@ static void sRSAKeyValueHandler(XMLElement &elem, void *userData)
 	*@retval E_SUCCESS otherwise
 	*@see getPublicKeyAsXML
 	*/
-SINT32 CAASymCipher::setPublicKeyAsXML(UINT8* key,UINT32 len)
+SINT32 CAASymCipher::setPublicKeyAsXML(const UINT8* key,UINT32 len)
 	{
 		if(key==NULL)
 			return E_UNKNOWN;
@@ -330,7 +341,7 @@ SINT32 CAASymCipher::setPublicKeyAsDOMNode(DOM_Node& node)
 										char* tmpStr=child.getFirstChild().getNodeValue().transcode();
 										decLen=4096;
 										CABase64::decode((UINT8*)tmpStr,strlen(tmpStr),decBuff,&decLen);
-										delete tmpStr;
+										delete[] tmpStr;
 										tmpRSA->n=BN_bin2bn(decBuff,decLen,NULL);
 									}
 								else if(child.getNodeName().equals("Exponent"))
@@ -340,7 +351,7 @@ SINT32 CAASymCipher::setPublicKeyAsDOMNode(DOM_Node& node)
 										char* tmpStr=child.getFirstChild().getNodeValue().transcode();
 										decLen=4096;
 										CABase64::decode((UINT8*)tmpStr,strlen(tmpStr),decBuff,&decLen);
-										delete tmpStr;
+										delete[] tmpStr;
 										tmpRSA->e=BN_bin2bn(decBuff,decLen,NULL);
 									}
 								child=child.getNextSibling();
@@ -350,6 +361,7 @@ SINT32 CAASymCipher::setPublicKeyAsDOMNode(DOM_Node& node)
 								if(m_pRSA!=NULL)
 									RSA_free(m_pRSA);
 								m_pRSA=tmpRSA;
+								setRSAFlags(m_pRSA);
 								return E_SUCCESS;
 							}
 						RSA_free(tmpRSA);
@@ -359,3 +371,25 @@ SINT32 CAASymCipher::setPublicKeyAsDOMNode(DOM_Node& node)
 			}
 		return E_UNKNOWN;
 	}
+
+/** Sets the public key which is used for encryption to the contained in the
+	*	provided certificate. The key has to be a RSA public key.
+	*	@retval E_SUCCESS if successful
+	*	@retval E_UNKNOWN otherwise (in this case the key leaves untouched)
+	*/
+SINT32 CAASymCipher::setPublicKey(const CACertificate* pCert)
+	{
+		if(pCert==NULL)
+			return E_UNKNOWN;
+		EVP_PKEY* pubkey=X509_get_pubkey(pCert->m_pCert);
+		if(pubkey==NULL||(pubkey->type!=EVP_PKEY_RSA&&pubkey->type!=EVP_PKEY_RSA2))
+			return E_UNKNOWN;
+		RSA* r=pubkey->pkey.rsa;
+		if(RSA_size(r)!=128)
+			return E_UNKNOWN;
+		if(m_pRSA!=NULL)
+			RSA_free(m_pRSA);
+		m_pRSA=r;
+		setRSAFlags(m_pRSA);
+		return E_SUCCESS;
+	}
