@@ -29,9 +29,12 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 #include "CAFirstMixChannelList.hpp"
 #include "CAUtil.hpp"
 #include "CAMsg.hpp"
-
 #define MAX_HASH_KEY 8113
 
+#ifdef PAYMENT
+#include "CAAccountingInstance.hpp"
+#endif
+
 CAFirstMixChannelList::CAFirstMixChannelList()
 	{
 		m_HashTable=new LP_fmHashTableEntry[MAX_HASH_KEY];
@@ -48,16 +51,40 @@ CAFirstMixChannelList::CAFirstMixChannelList()
 #ifdef DO_TRACE
 	m_aktAlloc=m_maxAlloc=0;
 #endif
+#ifdef DELAY_USERS
+		m_u32DelayChannelUnlimitTraffic=DELAY_USERS_TRAFFIC;
+		m_u32DelayChannelBucketGrow=DELAY_USERS_BUCKET_GROW;
+		m_u32DelayChannelBucketGrowIntervall=DELAY_USERS_BUCKET_GROW_INTERVALL;
+		m_pDelayBuckets=new UINT32*[MAX_POLLFD];
+		memset(m_pDelayBuckets,0,sizeof(UINT32*)*MAX_POLLFD);
+		m_pMutexDelayChannel=new CAMutex();
+		m_pThreadDelayBucketsLoop=new CAThread();
+		m_bDelayBucketsLoopRun=true;
+		m_pThreadDelayBucketsLoop->setMainLoop(fml_loopDelayBuckets);
+		m_pThreadDelayBucketsLoop->start(this);
+#endif
+
+//#ifdef PAYMENT
+//	m_pAccountingInstance = CAAccountingInstance::getInstance();
+//#endif
 	}
 
 CAFirstMixChannelList::~CAFirstMixChannelList()
 	{
+#ifdef DELAY_USERS
+		m_bDelayBucketsLoopRun=false;
+		m_pThreadDelayBucketsLoop->join();
+		delete m_pThreadDelayBucketsLoop;
+		delete m_pMutexDelayChannel;
+		delete []m_pDelayBuckets;
+#endif
 		for(int i=0;i<MAX_HASH_KEY;i++)
 				{
 					delete m_HashTable[i];
 				}
 		delete []m_HashTable;
 		delete []m_HashTableOutChannels;
+		
 	}
 		
 /** Adds a new TCP/IP connection (a new user) to the channel list.
@@ -67,7 +94,7 @@ CAFirstMixChannelList::~CAFirstMixChannelList()
 	* @retval E_UNKNOWN in case of an error
 	* @retval E_SUCCESS if successful
 	*/
-SINT32 CAFirstMixChannelList::add(CAMuxSocket* pMuxSocket,UINT8 peerIP[4],CAQueue* pQueueSend)
+SINT32 CAFirstMixChannelList::add(CAMuxSocket* pMuxSocket,const UINT8 peerIP[4],CAQueue* pQueueSend)
 	{
 		if(pMuxSocket==NULL)
 			return E_UNKNOWN;
@@ -83,14 +110,28 @@ SINT32 CAFirstMixChannelList::add(CAMuxSocket* pMuxSocket,UINT8 peerIP[4],CAQueu
 			}
 		pHashTableEntry->pMuxSocket=pMuxSocket;
 		pHashTableEntry->pQueueSend=pQueueSend;
+		pHashTableEntry->pControlChannelDispatcher=new CAControlChannelDispatcher(pQueueSend);
+		pHashTableEntry->uAlreadySendPacketSize=-1;
 		pHashTableEntry->cNumberOfChannels=0;
-#ifdef LOG_CHANNEL
+#ifdef LOG_TRAFFIC_PER_USER
 		pHashTableEntry->trafficIn=0;
 		pHashTableEntry->trafficOut=0;
 		getcurrentTimeMillis(pHashTableEntry->timeCreated);
 		getRandom((UINT8*)&pHashTableEntry->id,8);
 #endif
 		memcpy(pHashTableEntry->peerIP,peerIP,4);
+#ifdef DELAY_USERS
+		pHashTableEntry->delayBucket=m_u32DelayChannelUnlimitTraffic; //can always send some first packets
+		for(UINT32 i=0;i<MAX_POLLFD;i++)
+			{
+				if(m_pDelayBuckets[i]==NULL)
+					{
+						pHashTableEntry->delayBucketID=i;
+						break;
+					}
+			}
+		m_pDelayBuckets[pHashTableEntry->delayBucketID]=&pHashTableEntry->delayBucket;
+#endif
 
 		//now insert the new connection in the list of all open connections
 		if(m_listHashTableHead==NULL) //if first one
@@ -106,12 +147,19 @@ SINT32 CAFirstMixChannelList::add(CAMuxSocket* pMuxSocket,UINT8 peerIP[4],CAQueu
 				m_listHashTableHead->list_HashEntries.prev=pHashTableEntry;
 				m_listHashTableHead=pHashTableEntry;				
 			}
+#ifdef PAYMENT
+		// init accounting instance for this user
+		CAAccountingInstance *pAccInst;
+		pAccInst = CAAccountingInstance::getInstance();
+//		pAccInst->initTableEntry(pHashTableEntry);
+//		m_pAccountingInstance->initTableEntry(pHashTableEntry);
+#endif
 		m_Mutex.unlock();
 		return E_SUCCESS;
 	}
 
 ///The maximum number of channels allowed per connection
-#define MAX_NUMBER_OF_CHANNELS 50
+#define MAX_NUMBER_OF_CHANNELS CHANNELS_PER_CLIENT
 
 /** Adds a new channel for a given connection to the channel list. 
 	* Also a new out-channel id is generated and returned.
@@ -151,7 +199,7 @@ SINT32 CAFirstMixChannelList::addChannel(CAMuxSocket* pMuxSocket,HCHANNEL channe
 		do
 			{
 				getRandom(channelOut); //get new Random OUT-CHANNEL-ID
-			} while(*channelOut==DUMMY_CHANNEL||get_intern_without_lock(*channelOut)!=NULL); //until it is unused...
+			} while(*channelOut<256||get_intern_without_lock(*channelOut)!=NULL); //until it is unused...
 		pNewEntry->channelOut=*channelOut;
 		pNewEntry->bIsSuspended=false;
 		pNewEntry->pHead=pHashTableEntry;
@@ -159,7 +207,6 @@ SINT32 CAFirstMixChannelList::addChannel(CAMuxSocket* pMuxSocket,HCHANNEL channe
 #ifdef LOG_CHANNEL
 		pNewEntry->packetsInFromUser=0;
 		pNewEntry->packetsOutToUser=0;
-		getcurrentTimeMillis(pNewEntry->timeCreated);
 #endif
 
 		
@@ -192,6 +239,25 @@ SINT32 CAFirstMixChannelList::addChannel(CAMuxSocket* pMuxSocket,HCHANNEL channe
 		return E_SUCCESS;
 	}
 
+
+/** Returns the general data stored for a given Socket (user)
+	* @param pMuxSocket the connection from the user
+	* @return general data for the given user
+	* @retval NULL if not found
+*/
+fmHashTableEntry* CAFirstMixChannelList::get(CAMuxSocket* pMuxSocket)
+	{
+		if(pMuxSocket==NULL)
+			return NULL;
+		SINT32 hashkey=pMuxSocket->getSocket();
+		if(hashkey>MAX_HASH_KEY-1||hashkey<0)
+			return NULL;
+		m_Mutex.lock();
+		fmHashTableEntry* pHashTableEntry=m_HashTable[hashkey];
+		m_Mutex.unlock();
+		return pHashTableEntry;
+	}
+
 /** Returns the information for a given Input-Channel-ID
 	* @param pMuxSocket the connection from the user
 	* @param channelIn the channel id
@@ -221,8 +287,8 @@ fmChannelListEntry* CAFirstMixChannelList::get(CAMuxSocket* pMuxSocket,HCHANNEL
 		return NULL;		
 	}
 
-/** Removes all channels, which belongs to the given connection and the connection itself from the
-	* list.
+/** Removes all channels, which belongs to the given connection and 
+	* the connection itself from the list.
 	* @param pMuxSocket the connection from the user
 	* @retval E_SUCCESS if successful
 	* @retval E_UNKNOWN in case of an error
@@ -241,6 +307,13 @@ SINT32 CAFirstMixChannelList::remove(CAMuxSocket* pMuxSocket)
 				m_Mutex.unlock();
 				return E_UNKNOWN;
 			}
+	#ifdef DELAY_USERS
+		m_pMutexDelayChannel->lock();
+		m_pDelayBuckets[pHashTableEntry->delayBucketID]=NULL;
+		m_pMutexDelayChannel->unlock();
+	#endif
+		pHashTableEntry->pControlChannelDispatcher->deleteAllControlChannels();
+		delete pHashTableEntry->pControlChannelDispatcher; //deletes the dispatcher and all associated control channels
 		if(m_listHashTableNext==pHashTableEntry) //adjust the enumeration over all connections (@see getNext())
 			m_listHashTableNext=pHashTableEntry->list_HashEntries.next;
 		
@@ -316,6 +389,13 @@ SINT32 CAFirstMixChannelList::remove(CAMuxSocket* pMuxSocket)
 #endif
 				pEntry=pTmpEntry;
 			}
+#ifdef PAYMENT
+		// cleanup accounting information
+		CAAccountingInstance * pAccInst;
+		pAccInst = CAAccountingInstance::getInstance();
+		pAccInst->cleanupTableEntry(pHashTableEntry);
+	//	m_pAccountingInstance->cleanupTableEntry(pHashTableEntry);
+#endif
 		memset(pHashTableEntry,0,sizeof(fmHashTableEntry)); //'delete' the connection from the connection hash table 
 		m_Mutex.unlock();
 		return E_SUCCESS;
@@ -429,10 +509,12 @@ SINT32 CAFirstMixChannelList::removeChannel(CAMuxSocket* pMuxSocket,HCHANNEL cha
 	*/
 fmHashTableEntry* CAFirstMixChannelList::getFirst()
 	{
+		m_Mutex.lock();
 		if(m_listHashTableHead!=NULL)
 			m_listHashTableNext=m_listHashTableHead->list_HashEntries.next;
 		else
 			m_listHashTableNext=NULL;
+		m_Mutex.unlock();
 		return m_listHashTableHead;
 	}
 
@@ -443,9 +525,11 @@ fmHashTableEntry* CAFirstMixChannelList::getFirst()
 	*/
 fmHashTableEntry* CAFirstMixChannelList::getNext()
 	{
+		m_Mutex.lock();
 		fmHashTableEntry* tmpEntry=m_listHashTableNext;
 		if(m_listHashTableNext!=NULL)
 			m_listHashTableNext=m_listHashTableNext->list_HashEntries.next;
+		m_Mutex.unlock();
 		return tmpEntry;
 	}
 
@@ -486,7 +570,7 @@ SINT32 CAFirstMixChannelList::test()
 		((CASocket*)pMuxSocket)->create();
 		UINT8 peerIP[4];
 		pList->add(pMuxSocket,peerIP,NULL);
-#if defined(_WIN32) && defined(_DEBUG)
+#if defined(HAVE_CRTDBG)
 		_CrtMemState s1, s2, s3;
 		_CrtMemCheckpoint( &s1 );
 #endif
@@ -495,7 +579,7 @@ SINT32 CAFirstMixChannelList::test()
 			pList->addChannel(pMuxSocket,i,NULL,&channelOut);
 		for(i=0;i<50;i++)
 			pList->removeChannel(pMuxSocket,i);
-#if defined(_WIN32) && defined(_DEBUG)
+#if defined(HAVE_CRTDBG)
 		_CrtMemCheckpoint( &s2 );
 		if ( _CrtMemDifference( &s3, &s1, &s2 ) )
       _CrtMemDumpStatistics( &s3 );
@@ -506,3 +590,35 @@ SINT32 CAFirstMixChannelList::test()
 		delete pList;
 		return E_SUCCESS;
 	}
+	
+#ifdef DELAY_USERS
+	THREAD_RETURN fml_loopDelayBuckets(void* param)
+		{
+			CAFirstMixChannelList* pChannelList=(CAFirstMixChannelList*)param;
+			UINT32** pDelayBuckets=pChannelList->m_pDelayBuckets;
+			while(pChannelList->m_bDelayBucketsLoopRun)
+				{
+					pChannelList->m_pMutexDelayChannel->lock();
+					UINT32 u32BucketGrow=pChannelList->m_u32DelayChannelBucketGrow;
+					for(UINT32 i=0;i<MAX_POLLFD;i++)
+						if(pDelayBuckets[i]!=NULL)
+							*(pDelayBuckets[i])+=u32BucketGrow;
+					pChannelList->m_pMutexDelayChannel->unlock();		
+					msSleep(pChannelList->m_u32DelayChannelBucketGrowIntervall);
+				}
+			THREAD_RETURN_SUCCESS;
+		}
+		
+	void CAFirstMixChannelList::setDelayParameters(UINT32 unlimitTraffic,UINT32 bucketGrow,UINT32 intervall)
+		{
+			m_pMutexDelayChannel->lock();
+			m_u32DelayChannelUnlimitTraffic=unlimitTraffic;
+			m_u32DelayChannelBucketGrow=bucketGrow;
+			m_u32DelayChannelBucketGrowIntervall=intervall;
+			for(UINT32 i=0;i<MAX_POLLFD;i++)
+				if(m_pDelayBuckets[i]!=NULL)
+					*(m_pDelayBuckets[i])=m_u32DelayChannelUnlimitTraffic;
+			m_pMutexDelayChannel->unlock();		
+		}																												
+		
+#endif	
