@@ -41,32 +41,46 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 	#define LOG_INFO	2 
 	#define LOG_DEBUG 3
 #endif
-#define LOG_SPECIAL 255
+#define LOG_ENCRYPTED 255
 
 #include "CAMutex.hpp"
+#include "CASymCipher.hpp"
+
+struct S_LOGENCCIPHER
+{
+	UINT8 iv[16];
+	int iv_off;
+	AES_KEY oKey;
+};
+
+typedef S_LOGENCCIPHER t_LogEncCipher;
+
 class CAMsg
 	{
-		protected:
+		private:
 			CAMsg(); //Singleton!
-			static CAMsg oMsg;
+			static CAMsg* pMsg;
 		public:
 			~CAMsg();
+			static SINT32 init(){pMsg=new CAMsg();return E_SUCCESS;}
+			static SINT32 cleanup(){delete pMsg;return E_SUCCESS;}
 			static SINT32 setLogOptions(UINT32 options);
-			static SINT32 printMsg(UINT32 typ,char* format,...);
-			static SINT32 openSpecialLog();
-			static SINT32 closeSpecialLog();
-		protected:
+			static SINT32 printMsg(UINT32 typ,const char* format,...);
+			static SINT32 openEncryptedLog();
+			static SINT32 closeEncryptedLog();
+		private:
 			SINT32 openLog(UINT32 type);
 			SINT32 closeLog();
 			UINT32 m_uLogType;
-			int m_hFileSpecial;
+			int m_hFileEncrypted;
 			int m_hFileInfo;
 			char *m_strMsgBuff;
 			char *m_strLogFile; 
 			static const char* const m_strMsgTypes[5];
-			CAMutex m_csPrint;
+			CAMutex* m_pcsPrint;
 #ifdef COMPRESSED_LOGS
 			gzFile m_gzFileInfo;
 #endif
+			t_LogEncCipher* m_pCipher;
    };
 #endif
