@@ -28,9 +28,12 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 #include "StdAfx.h"
 #include "CASocket.hpp"
 #include "CASocketAddrINet.hpp"
+#ifdef HAVE_UNIX_DOMAIN_PROTOCOL
+	#include "CASocketAddrUnix.hpp"
+#endif
 #include "CASingleSocketGroup.hpp"
 #ifdef _DEBUG
-	extern int sockets;
+	int sockets;
 #endif
 #include "CAMsg.hpp"
 #include "CAUtil.hpp"
@@ -69,20 +72,40 @@ SINT32 CASocket::create(int type)
 		return E_SUCCESS;
 	}
 
+/** Starts listening on address psa.
+	* @retval E_SUCCESS, if successful
+	* @retval E_SOCKET_LISTEN, if call to listen() returns an error
+	* @retval E_SOCKET_BIND,	 if call to bind() returns an error
+	* @retval E_UNKNOWN, otherwise
+	*/
 SINT32 CASocket::listen(const CASocketAddr& psa)
 	{
-		int type=psa.getType();
+		SINT32 type=psa.getType();
 		if(m_bSocketIsClosed&&create(type)!=E_SUCCESS)
 			return E_UNKNOWN;
+#ifdef HAVE_UNIX_DOMAIN_PROTOCOL
+		//we have to delete the file before...
+		if(psa.getType()==AF_LOCAL)
+			{
+				UINT8* path=((CASocketAddrUnix&)psa).getPath();
+				if(path!=NULL)
+					{
+						SINT32 ret=::unlink((char*)path);
+						if(ret!=0)
+							CAMsg::printMsg(LOG_ERR,"CASocket::listen() -- could not unlink unix domain socket file name %s -- a call to bind or listen may fail...\n",path);
+						delete path;
+					}
+			}
+#endif			
 		if(::bind(m_Socket,psa.LPSOCKADDR(),psa.getSize())==SOCKET_ERROR)
 			{
 				close();
-				return E_UNKNOWN;
+				return E_SOCKET_BIND;
 			}
 		if(::listen(m_Socket,SOMAXCONN)==SOCKET_ERROR)
 			{
 				close();
-				return E_UNKNOWN;
+				return E_SOCKET_LISTEN;
 			}
 		return E_SUCCESS;
 	}
@@ -116,12 +139,6 @@ SINT32 CASocket::accept(CASocket &s)
 	}
 
 
-/** Tries to connect to the peer described by psa*/			
-SINT32 CASocket::connect(CASocketAddr & psa)
-	{
-		return connect(psa,1,0);
-	}
-
 /** Tries to connect to the peer described by psa.
 	* @param psa - peer
 	* @param retry - number of retries
@@ -138,7 +155,7 @@ SINT32 CASocket::connect(CASocketAddr & psa,UINT32 retry,UINT32 time)
 		sockets++;
 #endif
 		int err=0;
-		LPSOCKADDR addr=psa.LPSOCKADDR();
+		const SOCKADDR* addr=psa.LPSOCKADDR();
 		int addr_len=psa.getSize();
 		for(UINT32 i=0;i<retry;i++)
 			{
@@ -156,7 +173,7 @@ SINT32 CASocket::connect(CASocketAddr & psa,UINT32 retry,UINT32 time)
 						#ifdef _DEBUG
 							CAMsg::printMsg(LOG_DEBUG,"Cannot connect... retrying\n");
 						#endif						
-						sSleep(time);
+						sSleep((UINT16)time);
 					}
 				else
 						return E_SUCCESS;
@@ -165,10 +182,10 @@ SINT32 CASocket::connect(CASocketAddr & psa,UINT32 retry,UINT32 time)
 	}
 			
 
-/** Tries to connect to peeer psa.
+/** Tries to connect to peer psa.
 	*
 	*	@param psa - peer
-	* @param msTimeOut - abort after msTimeOut MilliSeconds
+	* @param msTimeOut - abort after msTimeOut milli seconds
 	*/
 SINT32 CASocket::connect(CASocketAddr & psa,UINT32 msTimeOut)
 	{
@@ -205,8 +222,11 @@ SINT32 CASocket::connect(CASocketAddr & psa,UINT32 msTimeOut)
 		fd_set readSet,writeSet;
 		FD_ZERO(&readSet);
 		FD_ZERO(&writeSet);
+		#pragma warning( push )
+		#pragma warning( disable : 4127 ) //Disable: Bedingter Ausdruck ist konstant
 		FD_SET(m_Socket,&readSet);
 		FD_SET(m_Socket,&writeSet);
+		#pragma warning( pop )
 		err=::select(m_Socket+1,&readSet,&writeSet,NULL,&tval);
 #else
 		struct pollfd opollfd;
@@ -255,6 +275,7 @@ SINT32 CASocket::close()
 		return ret;
 	}
 
+/* /// it seems that this function is never used (Bastian)
 SINT32 CASocket::close(UINT32 mode)
 	{
 //		EnterCriticalSection(&csClose);
@@ -276,13 +297,15 @@ SINT32 CASocket::close(UINT32 mode)
 //		LeaveCriticalSection(&csClose);
 		return ret;
 	}
+*/
 			
-/** Sends some data over the network. This may block, if socket is in blocking mode.
-	@param buff the buffer of data to send
-	@param len content length
-	@retval E_AGAIN if non blocking socket would block or a timeout was reached
-	@retval E_UNKNOWN if an error occured
-	@return number of bytes send
+/** Sends some data over the network. This may block, 
+	* if socket is in blocking mode.
+	* @param buff the buffer of data to send
+	* @param len content length
+	* @retval E_AGAIN if non blocking socket would block or a timeout was reached
+	* @retval E_UNKNOWN if an error occured
+	* @return number of bytes send
 */
 SINT32 CASocket::send(const UINT8* buff,UINT32 len)
 	{
@@ -295,20 +318,28 @@ SINT32 CASocket::send(const UINT8* buff,UINT32 len)
 				ret=::send(m_Socket,(char*)buff,len,MSG_NOSIGNAL);
 				#ifdef _DEBUG
 					if(ret==SOCKET_ERROR)
-						CAMsg::printMsg(LOG_DEBUG,"Fehler beim Socket-send: %i",GET_NET_ERROR);
+						CAMsg::printMsg(LOG_DEBUG,"Fehler beim Socket-send: %i\n",GET_NET_ERROR);
 				#endif
 			}
 		while(ret==SOCKET_ERROR&&(ef=GET_NET_ERROR)==EINTR);
 		if(ret==SOCKET_ERROR)
 			{
 				if(ef==ERR_INTERN_WOULDBLOCK)
-					return E_AGAIN;
+					{
+						#ifdef _DEBUG
+							CAMsg::printMsg(LOG_DEBUG,"Fehler beim Socket-send: E_AGAIN\n");
+						#endif
+						return E_AGAIN;
+					}
+				#ifdef _DEBUG
+					CAMsg::printMsg(LOG_DEBUG,"Fehler beim Socket-send:E_UNKNOWN (ef=%i)\n",ef);
+				#endif
 				return E_UNKNOWN;
 			}
 	  return ret;	    	    
 	}
 
-/** Sends some data over the network. Using a Timeout if socket is in blocking mode. 
+/** Sends some data over the network. Using a Timeout iff socket is in blocking mode. 
 	Otherwise E_AGAIN may returned
 	@param buff the buffer to send
 	@param len content length
@@ -316,6 +347,7 @@ SINT32 CASocket::send(const UINT8* buff,UINT32 len)
 	@retval E_AGAIN if Operation would block on a non-blocking socket
 	@retval E_TIMEDOUT if the timeout was reached
 	@return number of bytes send, or -1 in case of an error
+	///FIXME really buggy!!!!
 */
 SINT32 CASocket::sendTimeOut(const UINT8* buff,UINT32 len,UINT32 msTimeOut)
 	{
@@ -372,33 +404,40 @@ SINT32 CASocket::sendFully(const UINT8* buff,UINT32 len)
 					return E_SUCCESS;
 				else if(ret==E_AGAIN)
 					{
-						CASingleSocketGroup ossg;
-						ossg.add(*this);
-						ret=ossg.select(false,1000);
+						ret=CASingleSocketGroup::select_once(*this,true,1000);
 						if(ret>=0||ret==E_TIMEDOUT)
 							continue;
+						#ifdef _DEBUG
+							CAMsg::printMsg(LOG_DEBUG,"CASocket::sendFully() - error near select_once() ret=%i\n",ret);
+						#endif
 						return E_UNKNOWN;
 					}
 				else if(ret<0)
 					{
+						#ifdef _DEBUG
+							CAMsg::printMsg(LOG_DEBUG,"CASocket::sendFully() - send returned %i\n",ret);
+						#endif
 						return E_UNKNOWN;
 					}
 				len-=ret;
 				buff+=ret;
 			}
+			//could never be here....
 	}
 
 /** Will receive some bytes from the socket. May block or not depending on whatever this socket
 	* was set to blocking or non-blocking mode.
 	* Warning: If socket is in blocking mode and receive is called, receive will block until some
 	* data is available, EVEN IF AN OTHER THREAD WILL CLOSE THIS SOCKET!
+	*
 	* @param buff the buffer which get the received data
 	* @param len size of buff
 	*	@return SOCKET_ERROR if an error occured
-	* @retval E_AGAIN, if socket was in non-blocking mode and receive would block or a timeout was reached
+	* @retval E_AGAIN, if socket was in non-blocking mode and 
+	*                  receive would block or a timeout was reached
 	* @retval 0 if socket was gracefully closed
 	* @return the number of bytes received (always >0)
-**/
+***/
 SINT32 CASocket::receive(UINT8* buff,UINT32 len)
 	{
 		int ret;	
@@ -415,17 +454,15 @@ SINT32 CASocket::receive(UINT8* buff,UINT32 len)
 			}
 #ifdef _DEBUG
 		if(ret==SOCKET_ERROR)
-	      CAMsg::printMsg(LOG_DEBUG,"Receive error: %i\n",ef);
+	      CAMsg::printMsg(LOG_DEBUG,"CASocket Receive error %d (%s)\n",ef,GET_NET_ERROR_STR(ef));
 #endif
 	  return ret;	    	    
 	}
 
-/**Receives all bytes. This blocks until all bytes are received or an error occured.
-@return E_UNKNOWN, in case of an error
-@return E_SUCCESS otherwise
-
-//TODO:: bugy for non-blocking socket...
-*/
+/** Receives all len bytes. This blocks until all bytes are received or an error occured.
+	* @return E_UNKNOWN, in case of an error
+	* @return E_SUCCESS otherwise
+***/
 SINT32 CASocket::receiveFully(UINT8* buff,UINT32 len)
 	{
 		SINT32 ret;
@@ -433,9 +470,17 @@ SINT32 CASocket::receiveFully(UINT8* buff,UINT32 len)
 	  do
 			{
 				ret=receive(buff+pos,len);
+//#ifdef _DEBUG
+//				CAMsg::printMsg(LOG_DEBUG,"CASocket::receiveFully - call to receive returned: %i\n",ret);
+//#endif
 				if(ret<=0)
 					{
-						if(ret==0||(ret!=E_AGAIN&&ret!=E_TIMEDOUT))
+						if(ret==E_AGAIN)
+							{
+								msSleep(100);
+								continue;
+							}
+						else
 							return E_UNKNOWN;
 					}
 				pos+=ret;
@@ -445,47 +490,47 @@ SINT32 CASocket::receiveFully(UINT8* buff,UINT32 len)
 	  return E_SUCCESS;	    	    
 	}
 
-/** Trys to receive all bytes. After the timout value has elpased, the error E_TIMEDOUT is returned
-	* Would not work correctly on Windows....
+/** Trys to receive all bytes. If after the timeout value has elapsed, 
+	* not all bytes are received
+	* the error E_TIMEDOUT is returned.
 	*	@param buff byte array, where the received bytes would be stored 
 	*	@param len	on input holds the number of bytes which should be read,
-	*							on return gives the number of bytes which are read before the timeout
 	*	@param msTimeOut the timout in milli seconds
 	* @retval E_TIMEDOUT if not all byts could be read
 	* @retval E_UNKNOWN if an error occured
 	* @retval E_SUCCESS if all bytes could be read
 	*
-	* TODO: Lots of work TODO!!!!!
 	*/
-SINT32 CASocket::receiveFully(UINT8* buff,UINT32 len,UINT16 msTimeOut)
+SINT32 CASocket::receiveFully(UINT8* buff,UINT32 len,UINT32 msTimeOut)
 	{
 		SINT32 ret;
-		UINT16 dt=msTimeOut/2+1;
-		SINT32 timeoutLeft=msTimeOut;
-		do
+		UINT32 pos=0;
+		UINT64 currentTime,endTime;
+		getcurrentTimeMillis(currentTime);
+		set64(endTime,currentTime);
+		add64(endTime,msTimeOut);
+		CASingleSocketGroup oSG(false);
+		oSG.add(*this);
+		for(;;)
 			{
-				#ifdef HAVE_AVAILABLE
-					ret=available();
-				#else
-					ret=::recv(m_Socket,(char*)buff,len,MSG_NOSIGNAL|MSG_PEEK|MSG_DONTWAIT);
-				#endif		
-				if(ret<=0) //This may be a bug (=0 ?)
-					return SOCKET_ERROR;
-				if((UINT32)ret>=len)
+				ret=oSG.select(msTimeOut);
+				if(ret==1)
 					{
-						ret=receive(buff,len);
-						if(ret<=0||(UINT32)ret!=len)
-							{
-								CAMsg::printMsg(LOG_DEBUG,"ReceiveFully receive error ret=%i\n",ret);
-								return E_UNKNOWN;
-							}
-						return E_SUCCESS;	    	    
+						ret=receive(buff+pos,len);
+						if(ret<=0)
+							return E_UNKNOWN;
+						pos+=ret;
+						len-=ret;
 					}
-				msSleep(dt);
-				timeoutLeft-=dt;
+				else if(ret==E_TIMEDOUT)
+					return E_TIMEDOUT;
+				if(len==0)
+					return E_SUCCESS;
+				getcurrentTimeMillis(currentTime);
+				if(!isLesser64(currentTime,endTime))
+					return E_TIMEDOUT;
+				msTimeOut=diff64(endTime,currentTime);
 			}
-		while(timeoutLeft>0);
-		return E_TIMEDOUT;
 	}
 
 SINT32 CASocket::getLocalPort()
@@ -514,6 +559,7 @@ SINT32 CASocket::setReuseAddr(bool b)
 		return setsockopt(m_Socket,SOL_SOCKET,SO_REUSEADDR,(char*)&val,sizeof(val));
 	}
 
+/*
 SINT32 CASocket::setRecvLowWat(UINT32 r)
 	{
 		int val=r;
@@ -535,7 +581,7 @@ SINT32 CASocket::getSendLowWat()
 		else
 			return val;
 	}
-
+*/
 SINT32 CASocket::setRecvBuff(UINT32 r)
 	{
 		int val=r;
@@ -551,7 +597,7 @@ SINT32 CASocket::getRecvBuff()
 		else
 			return val;
 	}
-/** Returuns < 0 on error, otherwise the new sendbuffersize (which may be less than r)*/
+/** Returns < 0 on error, otherwise the new sendbuffersize (which may be less than r)*/
 SINT32 CASocket::setSendBuff(SINT32 r)
 	{
 		if(r<0)
